@@ -1,18 +1,7 @@
 //
 // Copyright 2014-17 Ettus Research, A National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 /***********************************************************************
@@ -27,17 +16,17 @@
 #include <uhd/usrp/dboard_manager.hpp>
 #include <uhd/utils/assert_has.hpp>
 #include <uhd/utils/log.hpp>
-#include <uhd/utils/msg.hpp>
 #include <uhd/utils/static.hpp>
 #include <uhd/utils/safe_call.hpp>
-#include <boost/assign/list_of.hpp>
+#include <uhdlib/usrp/common/max287x.hpp>
+
 #include <boost/shared_ptr.hpp>
 #include <boost/math/special_functions/round.hpp>
-#include <boost/thread.hpp>
 #include <boost/algorithm/string.hpp>
 #include <boost/thread/mutex.hpp>
 #include <map>
-#include "max287x.hpp"
+#include <chrono>
+#include <thread>
 
 using namespace uhd;
 using namespace uhd::usrp;
@@ -88,7 +77,8 @@ enum ubx_cpld_field_id_t
     RXDRV_FORCEON = 21,
     RXAMP_FORCEON = 22,
     RXLNA1_FORCEON = 23,
-    RXLNA2_FORCEON = 24
+    RXLNA2_FORCEON = 24,
+    CAL_ENABLE = 25
 };
 
 struct ubx_gpio_field_info_t
@@ -165,12 +155,12 @@ static const dboard_id_t UBX_TDD_160MHZ_RX_ID(0x0203);
 static const freq_range_t ubx_freq_range(10e6, 6.0e9);
 static const gain_range_t ubx_tx_gain_range(0, 31.5, double(0.5));
 static const gain_range_t ubx_rx_gain_range(0, 31.5, double(0.5));
-static const std::vector<std::string> ubx_pgas = boost::assign::list_of("PGA-TX")("PGA-RX");
-static const std::vector<std::string> ubx_plls = boost::assign::list_of("TXLO")("RXLO");
-static const std::vector<std::string> ubx_tx_antennas = boost::assign::list_of("TX/RX")("CAL");
-static const std::vector<std::string> ubx_rx_antennas = boost::assign::list_of("TX/RX")("RX2")("CAL");
-static const std::vector<std::string> ubx_power_modes = boost::assign::list_of("performance")("powersave");
-static const std::vector<std::string> ubx_xcvr_modes = boost::assign::list_of("FDX")("TX")("TX/RX")("RX");
+static const std::vector<std::string> ubx_pgas{"PGA-TX", "PGA-RX"};
+static const std::vector<std::string> ubx_plls{"TXLO", "RXLO"};
+static const std::vector<std::string> ubx_tx_antennas{"TX/RX", "CAL"};
+static const std::vector<std::string> ubx_rx_antennas{"TX/RX", "RX2", "CAL"};
+static const std::vector<std::string> ubx_power_modes{"performance", "powersave"};
+static const std::vector<std::string> ubx_xcvr_modes{"FDX", "TX", "TX/RX", "RX"};
 
 static const ubx_gpio_field_info_t ubx_proto_gpio_info[] = {
     //Field         Unit                  Offset Mask      Width    Direction                   ATR    IDLE,TX,RX,FDX
@@ -324,7 +314,7 @@ public:
             {
                 std::vector<double> rates = _iface->get_clock_rates(dboard_iface::UNIT_RX);
                 double highest_rate = 0.0;
-                BOOST_FOREACH(double rate, rates)
+                for(double rate:  rates)
                 {
                     if (rate <= pfd_freq_max and rate > highest_rate)
                         highest_rate = rate;
@@ -332,7 +322,9 @@ public:
                 try {
                     _iface->set_clock_rate(dboard_iface::UNIT_RX, highest_rate);
                 } catch (const uhd::not_implemented_error &) {
-                    UHD_MSG(warning) << "Unable to set dboard clock rate - phase will vary" << std::endl;
+                    UHD_LOG_WARNING("UBX",
+                        "Unable to set dboard clock rate - phase will vary"
+                    );
                     can_set_clock_rate = false;
                 }
                 _rx_target_pfd_freq = highest_rate;
@@ -341,7 +333,7 @@ public:
             {
                 std::vector<double> rates = _iface->get_clock_rates(dboard_iface::UNIT_TX);
                 double highest_rate = 0.0;
-                BOOST_FOREACH(double rate, rates)
+                for(double rate:  rates)
                 {
                     if (rate <= pfd_freq_max and rate > highest_rate)
                         highest_rate = rate;
@@ -349,7 +341,9 @@ public:
                 try {
                     _iface->set_clock_rate(dboard_iface::UNIT_TX, highest_rate);
                 } catch (const uhd::not_implemented_error &) {
-                    UHD_MSG(warning) << "Unable to set dboard clock rate - phase will vary" << std::endl;
+                    UHD_LOG_WARNING("UBX",
+                        "Unable to set dboard clock rate - phase will vary"
+                    );
                 }
                 _tx_target_pfd_freq = highest_rate;
             }
@@ -390,7 +384,7 @@ public:
         _iface->set_pin_ctrl(dboard_iface::UNIT_RX, _rx_gpio_reg.atr_mask);
 
         // bring CPLD out of reset
-        boost::this_thread::sleep(boost::posix_time::milliseconds(20)); // hold CPLD reset for minimum of 20 ms
+        std::this_thread::sleep_for(std::chrono::milliseconds(20)); // hold CPLD reset for minimum of 20 ms
 
         set_gpio_field(CPLD_RST_N, 1);
         write_gpio();
@@ -402,8 +396,8 @@ public:
             _txlo2 = max287x_iface::make<max2870>(boost::bind(&ubx_xcvr::write_spi_regs, this, TXLO2, _1));
             _rxlo1 = max287x_iface::make<max2870>(boost::bind(&ubx_xcvr::write_spi_regs, this, RXLO1, _1));
             _rxlo2 = max287x_iface::make<max2870>(boost::bind(&ubx_xcvr::write_spi_regs, this, RXLO2, _1));
-            std::vector<max287x_iface::sptr> los = boost::assign::list_of(_txlo1)(_txlo2)(_rxlo1)(_rxlo2);
-            BOOST_FOREACH(max287x_iface::sptr lo, los)
+            std::vector<max287x_iface::sptr> los{_txlo1, _txlo2, _rxlo1, _rxlo2};
+            for(max287x_iface::sptr lo:  los)
             {
                 lo->set_auto_retune(false);
                 lo->set_muxout_mode(max287x_iface::MUXOUT_DLD);
@@ -416,8 +410,8 @@ public:
             _txlo2 = max287x_iface::make<max2871>(boost::bind(&ubx_xcvr::write_spi_regs, this, TXLO2, _1));
             _rxlo1 = max287x_iface::make<max2871>(boost::bind(&ubx_xcvr::write_spi_regs, this, RXLO1, _1));
             _rxlo2 = max287x_iface::make<max2871>(boost::bind(&ubx_xcvr::write_spi_regs, this, RXLO2, _1));
-            std::vector<max287x_iface::sptr> los = boost::assign::list_of(_txlo1)(_txlo2)(_rxlo1)(_rxlo2);
-            BOOST_FOREACH(max287x_iface::sptr lo, los)
+            std::vector<max287x_iface::sptr> los{_txlo1, _txlo2, _rxlo1, _rxlo2};
+            for(max287x_iface::sptr lo:  los)
             {
                 lo->set_auto_retune(false);
                 //lo->set_cycle_slip_mode(true);  // tried it - caused longer lock times
@@ -480,7 +474,7 @@ public:
         get_tx_subtree()->create<std::vector<std::string> >("antenna/options")
             .set(ubx_tx_antennas);
         get_tx_subtree()->create<std::string>("antenna/value")
-            .add_coerced_subscriber(boost::bind(&ubx_xcvr::set_tx_ant, this, _1))
+            .set_coercer(boost::bind(&ubx_xcvr::set_tx_ant, this, _1))
             .set(ubx_tx_antennas.at(0));
         get_tx_subtree()->create<std::string>("connection")
             .set("QI");
@@ -517,7 +511,7 @@ public:
         get_rx_subtree()->create<std::vector<std::string> >("antenna/options")
             .set(ubx_rx_antennas);
         get_rx_subtree()->create<std::string>("antenna/value")
-            .add_coerced_subscriber(boost::bind(&ubx_xcvr::set_rx_ant, this, _1)).set("RX2");
+            .set_coercer(boost::bind(&ubx_xcvr::set_rx_ant, this, _1)).set("RX2");
         get_rx_subtree()->create<std::string>("connection")
             .set("IQ");
         get_rx_subtree()->create<bool>("enabled")
@@ -581,7 +575,7 @@ private:
     {
         boost::mutex::scoped_lock lock(_spi_mutex);
         ROUTE_SPI(_iface, dest);
-        BOOST_FOREACH(uint32_t value, values)
+        for(uint32_t value:  values)
             WRITE_SPI(_iface, value);
     }
 
@@ -741,14 +735,17 @@ private:
         return sensor_value_t("Unknown", false, "locked", "unlocked");
     }
 
-    void set_tx_ant(const std::string &ant)
+    std::string set_tx_ant(const std::string &ant)
     {
         //validate input
         assert_has(ubx_tx_antennas, ant, "ubx tx antenna name");
+        set_cpld_field(CAL_ENABLE, (ant == "CAL"));
+        write_cpld_reg();
+        return ant;
     }
 
     // Set RX antennas
-    void set_rx_ant(const std::string &ant)
+    std::string set_rx_ant(const std::string &ant)
     {
         boost::mutex::scoped_lock lock(_mutex);
         //validate input
@@ -771,6 +768,8 @@ private:
         }
         write_gpio();
         write_cpld_reg();
+
+        return ant;
     }
 
     /***********************************************************************
@@ -784,7 +783,7 @@ private:
         _ubx_tx_atten_val = ((attn_code & 0x3F) << 10);
         set_gpio_field(TX_GAIN, attn_code);
         write_gpio();
-        UHD_LOGV(rarely) << boost::format("UBX TX Gain: %f dB, Code: %d, IO Bits 0x%04x") % gain % attn_code % _ubx_tx_atten_val << std::endl;
+        UHD_LOGGER_TRACE("UBX") << boost::format("UBX TX Gain: %f dB, Code: %d, IO Bits 0x%04x") % gain % attn_code % _ubx_tx_atten_val ;
         _tx_gain = gain;
         return gain;
     }
@@ -797,7 +796,7 @@ private:
         _ubx_rx_atten_val = ((attn_code & 0x3F) << 10);
         set_gpio_field(RX_GAIN, attn_code);
         write_gpio();
-        UHD_LOGV(rarely) << boost::format("UBX RX Gain: %f dB, Code: %d, IO Bits 0x%04x") % gain % attn_code % _ubx_rx_atten_val << std::endl;
+        UHD_LOGGER_TRACE("UBX") << boost::format("UBX RX Gain: %f dB, Code: %d, IO Bits 0x%04x") % gain % attn_code % _ubx_rx_atten_val ;
         _rx_gain = gain;
         return gain;
     }
@@ -821,18 +820,18 @@ private:
         property_tree::sptr subtree = this->get_tx_subtree();
         device_addr_t tune_args = subtree->access<device_addr_t>("tune_args").get();
         is_int_n = boost::iequals(tune_args.get("mode_n",""), "integer");
-        UHD_LOGV(rarely) << boost::format("UBX TX: the requested frequency is %f MHz") % (freq/1e6) << std::endl;
+        UHD_LOGGER_TRACE("UBX") << boost::format("UBX TX: the requested frequency is %f MHz") % (freq/1e6) ;
         double target_pfd_freq = _tx_target_pfd_freq;
         if (is_int_n and tune_args.has_key("int_n_step"))
         {
             target_pfd_freq = tune_args.cast<double>("int_n_step", _tx_target_pfd_freq);
             if (target_pfd_freq > _tx_target_pfd_freq)
             {
-                UHD_MSG(warning)
+                UHD_LOGGER_WARNING("UBX")
                     << boost::format("Requested int_n_step of %f MHz too large, clipping to %f MHz")
                     % (target_pfd_freq/1e6)
                     % (_tx_target_pfd_freq/1e6)
-                    << std::endl;
+                    ;
                 target_pfd_freq = _tx_target_pfd_freq;
             }
         }
@@ -960,7 +959,7 @@ private:
         _txlo1_freq = freq_lo1;
         _txlo2_freq = freq_lo2;
 
-        UHD_LOGV(rarely) << boost::format("UBX TX: the actual frequency is %f MHz") % (_tx_freq/1e6) << std::endl;
+        UHD_LOGGER_TRACE("UBX") << boost::format("UBX TX: the actual frequency is %f MHz") % (_tx_freq/1e6) ;
 
         return _tx_freq;
     }
@@ -973,7 +972,7 @@ private:
         double ref_freq = _iface->get_clock_rate(dboard_iface::UNIT_RX);
         bool is_int_n = false;
 
-        UHD_LOGV(rarely) << boost::format("UBX RX: the requested frequency is %f MHz") % (freq/1e6) << std::endl;
+        UHD_LOGGER_TRACE("UBX") << boost::format("UBX RX: the requested frequency is %f MHz") % (freq/1e6) ;
 
         property_tree::sptr subtree = this->get_rx_subtree();
         device_addr_t tune_args = subtree->access<device_addr_t>("tune_args").get();
@@ -984,11 +983,11 @@ private:
             target_pfd_freq = tune_args.cast<double>("int_n_step", _rx_target_pfd_freq);
             if (target_pfd_freq > _rx_target_pfd_freq)
             {
-                UHD_MSG(warning)
+                UHD_LOGGER_WARNING("UBX")
                     << boost::format("Requested int_n_step of %f Mhz too large, clipping to %f MHz")
                     % (target_pfd_freq/1e6)
                     % (_rx_target_pfd_freq/1e6)
-                    << std::endl;
+                    ;
                 target_pfd_freq = _rx_target_pfd_freq;
             }
         }
@@ -1156,7 +1155,7 @@ private:
         _rxlo1_freq = freq_lo1;
         _rxlo2_freq = freq_lo2;
 
-        UHD_LOGV(rarely) << boost::format("UBX RX: the actual frequency is %f MHz") % (_rx_freq/1e6) << std::endl;
+        UHD_LOGGER_TRACE("UBX") << boost::format("UBX RX: the actual frequency is %f MHz") % (_rx_freq/1e6) ;
 
         return _rx_freq;
     }
